//+------------------------------------------------------------------+
//|                                                      News EA.mq4 |
//|                                Copyright 2013, Maarten van Hooft |
//|                                        Maarten_van_Hooft@live.nl |
//+------------------------------------------------------------------+

// This EA should automatically download the News. At times the news 
// is going to be released, download the .csv economic news file at 
// every second, until it is imported into the EA. This file will be
// uploaded with the use of 'GetRight' software, which automatically
// can download (on startup?). The pending order's relevants data
// will be plotted on the screen.

// At the init phase of the EA, Manually should be put in what needs to
// happen in comparison to the news. So a manual analysis should be
// made. This is an example:
   // If ActualNews1 =<2.0%    --> Sell: SellThreshold=2.0
   // If ActualNews1 =>2.5%    --> Buy:  BuyTrheshold=2.5
   
// Magic Numbers for this serie:
// News 1: 11
// News 2: 12
// News 3: 13
// News 4: 14
// News 5: 15

// Economic data retrieved from:
// www.dailyfx.com/files/Calendar-07-14-2013.csv

// Data is displayed as following:
// Date,Time,Time Zone,Currency,Event,Importance,Actual,Forecast,Previous
// Tue Jul 23,12:30,GMT,cad,CAD Retail Sales (MoM),Medium,,0.4%,0.1%

// Date        Mon Jul 23
// Time        12:30
// Time Zone   GMT
// Currency    CAD
// Event       CAD Retail Sales (MoM)
// Importance  Medium
// Actual    
// Forecast    0.4%
// Previous    0.1%

//+------------------------------------------------------------------+
//| Current Issues to be fixed                                       |
//+------------------------------------------------------------------+

// An issue is that sometimes, a number comes out with a non-alphabetical 
// character: "3.88B", or 2.53%. These strings which are in all of 
// Previous, Consensus, and Forecast strings. To be fixed.

//+------------------------------------------------------------------+
//| EA variables                                                     |
//+------------------------------------------------------------------+
#include <stdlib.mqh>
#import "kernel32.dll" //Necessary for getright launch

// External variables pending trade
extern string HtmlAdress="www.dailyfx.com/files/Calendar-07-21-2013.csv";
extern string Event="Fill in economic event"; //StringFind function
extern string ReleaseTimeGMT="01.08.2013 17:00";
extern double SellTreshold=0; // <=
extern double BuyTreshold=0; // >=
extern double Lots=0.03;
extern int TrailingStop=50;
extern int TakeProfit=50;
extern int StopLoss=50;
extern bool PivotStop=false; // If false, Pivot will not be taken as a StopLoss
extern string PivotSL="S1";
extern bool PivotTake=false; // If false, Pivot will not be taken as a TP
extern string PivotTP="R2";

// Internal variables
bool StopEA=false,NewSL=false,NewsReleased=false;
int Magic=11,Slippage=50,first=1,i;
double StopLossLevel,TakeProfitLevel;
double S3,S2,S1,P,R1,R2,R3;

// Variables for the calendar
string GetrightAdress = "c:\progra~1\getright\getright.exe";
string NewsDate="",NewsTime="",NewsTimeZone="",NewsCurrency="",NewsEvent="",NewsImportance="",NewsActual="",NewsForecast="",NewsPrevious="";
string NewsArray[,7];    
string ColourImpact,Today;
       
int WinExec(string NameEx, int dwFlags);
int Max=0,TotalEventsToday=0;
datetime NewsReleaseTime;

//+------------------------------------------------------------------+
//| expert initialization function                                   |
//+------------------------------------------------------------------+
int init()
  { 
// Start out clean
   ObjectsDeleteAll();

// Create Minutes object
   GrabNews();
    
// Determine PivotLevels
   PivotLevels();
   
// End Init function
   return(0);
  }

//+------------------------------------------------------------------+
//| expert start function                                            |
//+------------------------------------------------------------------+
int start()
  {
// If it is 2201 GMT, recalculate Pivot Levels.
  if(Hour()==22 && Minute()==01)
   PivotLevels();
   
// If it is time for the news to be released, download it every tick
// starting five seconds prior to the release
  if(TimeCurrent()+5>=NewsReleaseTime && NewsReleased==false)
    GrabNews();
   
// Check the actual information and make an action on it
  if(NewsReleased==true)
    Check();
    
// If there are positions open for this symbol, manage them   
  if(PositionsSymbol()>0)
   OrderManagement();     
   
  return(0);    
  }

//+------------------------------------------------------------------+
//| Download calendar function: If the settings in the GetRight.exe  |
//| program are correct, the program should download the calendar    |
//| into C:/Program~1/Broker/expert/files -specified in Getright.exe |
//+------------------------------------------------------------------+
//| This function reads and converts the data from the saved file    |
//| into information. This information should be converted into this |
//| Format displayed on the chart                                    |
//| Date|Time|TimeZone|Currency|Event|Imp.|Actual|Cons.|Previous|    |
//+------------------------------------------------------------------+
void GrabNews() 
  {
// Download calendar  
   WinExec(GetrightAdress+" /URL:"+HtmlAdress+" /FILE:Calendar.csv /W /O",0);
 
// Opens the file: from now on we are in the file
   int handle = FileOpen("Calendar.csv",FILE_READ|FILE_CSV,';');
   if (handle==-1||FileSize(handle)==0)
       Print("Error is",GetLastError());
       
// Select the order using StringFind    
   while(StringFind(NewsEvent,Event,0)==false)
   {
     NewsDate = FileReadString(handle);
     NewsTime = FileReadString(handle);
     NewsTimeZone = FileReadString(handle);
     NewsCurrency = FileReadString(handle);
     NewsEvent = FileReadString(handle);
     NewsImportance = FileReadString(handle);
     NewsActual = FileReadString(handle);
     NewsForecast = FileReadString(handle);
     NewsPrevious = FileReadString(handle);   
   }
            Print("Error is",GetLastError());
  if(first==1)
   {    
//Create Event Label   
    ObjectCreate("Event", OBJ_LABEL, 0, 0, 0);
    ObjectSetText("Event",NewsEvent,12, "Times New Roman", White);
    ObjectSet("Event", OBJPROP_CORNER, 0);
    ObjectSet("Event", OBJPROP_XDISTANCE, 10);
    ObjectSet("Event", OBJPROP_YDISTANCE, 15);    
    
//Create Time Label   
    ObjectCreate("Time", OBJ_LABEL, 0, 0, 0);
    ObjectSetText("Time","Rls: " +NewsDate+" "+NewsTime+" "+NewsTimeZone,12, "Times New Roman", White);
    ObjectSet("Time", OBJPROP_CORNER, 0);
    ObjectSet("Time", OBJPROP_XDISTANCE, 10);
    ObjectSet("Time", OBJPROP_YDISTANCE, 30);
        
//Create Impact label 
    ObjectCreate("Impact", OBJ_LABEL, 0, 0, 0);
    ObjectSetText("Impact","Imp: " +NewsImportance,12, "Times New Roman", White);
    ObjectSet("Impact", OBJPROP_CORNER, 0);
    ObjectSet("Impact", OBJPROP_XDISTANCE, 10);
    ObjectSet("Impact", OBJPROP_YDISTANCE, 45);      
    
//Create Currency Label   
    ObjectCreate("Cur", OBJ_LABEL, 0, 0, 0);
    ObjectSetText("Cur","Cur: " +NewsCurrency,12, "Times New Roman", White);
    ObjectSet("Cur", OBJPROP_CORNER, 0);
    ObjectSet("Cur", OBJPROP_XDISTANCE, 10);
    ObjectSet("Cur", OBJPROP_YDISTANCE, 60);          

//Create Previous Label
    ObjectCreate("Previous", OBJ_LABEL, 0, 0, 0);
    ObjectSetText("Previous","Prv: " +NewsPrevious,12, "Times New Roman", White);
    ObjectSet("Previous", OBJPROP_CORNER, 0);
    ObjectSet("Previous", OBJPROP_XDISTANCE, 10);
    ObjectSet("Previous", OBJPROP_YDISTANCE, 75);  

//Create Consensus Label
    ObjectCreate("Consensus", OBJ_LABEL, 0, 0, 0);
    ObjectSetText("Consensus","Cns: " +NewsForecast,12, "Times New Roman", White);
    ObjectSet("Consensus", OBJPROP_CORNER, 0);
    ObjectSet("Consensus", OBJPROP_XDISTANCE, 10);
    ObjectSet("Consensus", OBJPROP_YDISTANCE, 90);  
         
//Create Actual Label
    ObjectCreate("Actual", OBJ_LABEL, 0, 0, 0);
    ObjectSetText("Actual","Act: " +NewsActual,12, "Times New Roman", White);
    ObjectSet("Actual", OBJPROP_CORNER, 0);
    ObjectSet("Actual", OBJPROP_XDISTANCE, 10);
    ObjectSet("Actual", OBJPROP_YDISTANCE, 105);  
                           
    first=0;
   }
       Print("Error is",GetLastError());
// Update actual in case it has changed:
    ObjectSetText("Actual","Act: " +NewsActual,12, "Times New Roman", White);
   
    FileClose(handle);     
           Print("Error is",GetLastError()); 	     
   }  

//+------------------------------------------------------------------+
//| Calculate Pivot Levels                                           |
//+------------------------------------------------------------------+
void PivotLevels()
   {
// Determine Pivot Levels if Time==2201 GMT and at initalisation
    S3=iCustom(NULL,0,"PivotPointer",0,0);
    S2=iCustom(NULL,0,"PivotPointer",1,0);
    S1=iCustom(NULL,0,"PivotPointer",2,0);
    P=iCustom(NULL,0,"PivotPointer",3,0);
    R1=iCustom(NULL,0,"PivotPointer",4,0);      
    R2=iCustom(NULL,0,"PivotPointer",5,0);         
    R3=iCustom(NULL,0,"PivotPointer",6,0); 
   }

//+------------------------------------------------------------------+
//| Calculate amount of Positions for current symbol                 |
//+------------------------------------------------------------------+
double PositionsSymbol()
   {
   int Amount=0;
   
   for (i=0; i<=OrdersTotal(); i++)       
      { 
      OrderSelect(i,SELECT_BY_POS);
      if( OrderSymbol()==Symbol() )   
         {      
          Amount++;
         }   
      }   

   return(Amount);
   }

//+------------------------------------------------------------------+
//| Check for signals                                                |
//+------------------------------------------------------------------+
double Check()
   { 
// Open Price
   if(1>=BuyTreshold)
      {   
       StopEA=true;    
                 
       while(PositionsSymbol()<1)
         OrderSend(Symbol(),OP_BUY,Lots,Ask,Slippage,0,0,"",Magic,0,White);                                           
      }
           
// Open Price
   if(1<=SellTreshold)
      {   
       StopEA=true;    
                 
       while(PositionsSymbol()<1)
         OrderSend(Symbol(),OP_BUY,Lots,Ask,Slippage,0,0,"",Magic,0,White);                                           
      }
                                   

   return(0);
   }

//+------------------------------------------------------------------+
//| Order Management Function                                        |
//+------------------------------------------------------------------+
double OrderManagement()
   {

//SL TP Trendline Level
if(NewSL==false)
   {
    TakeProfitLevel=0;
   }    
 
if(NewSL==true)
   {
    //Move SL to BE
   }   

 for (i=0; i<=OrdersTotal(); i++)       
   { 
    OrderSelect(i,SELECT_BY_POS);
    if(OrderSymbol()==Symbol() && OrderMagicNumber()==Magic)
   
//If order with correct Magic no is selected, manage it
    {                                        
     if(OrderType()==0) // Type = Buy
            {            
            //Position TP     
            if( (Bid>=R1 && PivotTake==true) || (Bid>= TakeProfitLevel && TakeProfit!=0) )
               {
                Print("TP Buy reached");
                OrderClose(OrderTicket(),Lots,Bid,Slippage,Green);
               }                   
               
           //Position SL                
            else if(Bid==1)
               {         
                Print("SLBuy reached");         
                OrderClose(OrderTicket(),Lots,Bid,Slippage,Green);              
               }
                                
           }   
           
     else  {              
            if( (Bid<=S1 && PivotStop==true) || (Bid<= StopLossLevel && StopLoss!=0) )
               { 
                Print("TP Sell reached");               
                OrderClose(OrderTicket(),Lots,Ask,Slippage,Green);
               } 
               
            else if(Bid==1)
               {
                Print("SL Sell reached"); 
                OrderClose(OrderTicket(),Lots,Ask,Slippage,Green);
               }                 
           }  
     }
    //End for loop 
    }
   return(0);
  }                


//+------------------------------------------------------------------+
//| expert deinit function                                           |
//+------------------------------------------------------------------+
int deinit()
  {
    //Try to close all positions while shutting down
   while(PositionsSymbol()>0)
    {
    for (i=0; i<=OrdersTotal(); i++)       
     { 
     OrderSelect(i,SELECT_BY_POS);
     if(OrderSymbol()==Symbol())
       { 
       if(OrderType()==0)
         OrderClose(OrderTicket(),Lots,Bid,Slippage,Green);
            
       else
         OrderClose(OrderTicket(),Lots,Ask,Slippage,Green);
       }
     }  
   }
   ObjectsDeleteAll();
	 	 	   
    return(0);
  }

